package org.netkernel.nkse.search.endpoint;

import java.util.HashMap;
import java.util.Map;

import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class SearchIndexAccessor extends StandardAccessorImpl
{
	public SearchIndexAccessor()
	{	this.declareThreadSafe();
	}

	@Override
	public void onSource(INKFRequestContext context) throws Exception
	{	String type=context.getThisRequest().getArgumentValue("activeType");
		if(type.equals("nkseIndex"))
		{	
			context.logFormatted(INKFRequestContext.LEVEL_INFO, "MSG_SEARCH_INDEX", (Object)null);
			INKFRequest req=context.createRequest("active:spaceAggregateHDS");
			req.addArgument("uri", "res:/etc/system/SearchIndexEngine.xml");
			req.setRepresentationClass(IHDSNode.class);
			IHDSNode engines=(IHDSNode)context.issueRequest(req);
			
			HDSBuilder b=new HDSBuilder();
			b.pushNode("batch");
				IHDSNodeList nl=engines.getNodes("//config");
				for(IHDSNode n: nl)
				{	String identifier=(String)n.getFirstValue("indexer");
					n=n.getParent();
					String space=(String)n.getFirstValue("id");
					String version=(String)n.getFirstValue("version");
					try
					{	req=context.createRequest("active:wormhole");
						req.addArgument("space", space);
						req.addArgument("version", version);
						req.addArgument("uri", identifier);
						req.setRepresentationClass(IHDSNode.class);
						IHDSNode rep=(IHDSNode)context.issueRequest(req);
						b.importChildren(rep);
					}
					catch(NKFException e)
					{	context.logRaw(context.LEVEL_WARNING, "Failure requesting indexer "+space+" "+version+" "+identifier+" "+e.getDeepestId()+" "+e.getDeepestMessage());
					}
				}
 			b.popNode();
			
			req=context.createRequest("active:textIndexBatch");
			req.addArgumentByValue("operand", b.getRoot());
			context.createResponseFrom(context.issueRequestForResponse(req));
		}
		else if(type.equals("nkseIndexDelete"))
		{	INKFRequest req=context.createRequest("active:textIndexDelete");
			context.createResponseFrom(context.issueRequestForResponse(req));
		}
		else if(type.equals("nkseSearch"))
		{	String search=	context.source("arg:operand", String.class);
			if(!(search.indexOf(":")>0))
			{	//A basic search so construct a weighted query
				search="default:("+search+")  title:("+search+"^2)  desc:("+search+") keywords:("+search+")";
			}
			INKFRequest req=context.createRequest("active:textSearch");
			req.addArgumentByValue("operand", search);
			req.addArgument("limit", "1000000");
			req.setRepresentationClass(IHDSNode.class);
			IHDSNode rep=(IHDSNode)context.issueRequest(req);
			//Normalize on resolverID+documentID
			Map m=new HashMap();
			HDSBuilder b=new HDSBuilder();
			b.pushNode("hits");
			b.importNode(rep.getFirstNode("/hits/time"));
			b.importNode(rep.getFirstNode("/hits/total"));
			for(IHDSNode hit : rep.getNodes("/hits/hit"))
			{	String documentID=null;
				String resolverID=null;
				for(IHDSNode field: hit.getNodes("field"))
				{	String name=(String)field.getFirstValue("name");
					if(name.equals("documentID"))
					{	documentID=(String)field.getFirstValue("value");
					}
					else if(name.equals("resolverID"))
					{	resolverID=(String)field.getFirstValue("value");
					}
				}
				String key=resolverID+documentID;
				if(m.get(key)==null)
				{	m.put(key, Boolean.TRUE);
					b.importNode(hit);
				}
			}
			b.popNode();
			context.createResponseFrom(b.getRoot());
		}
	}
	
	private class NormalizedIdentifier
	{	
		public String mID;
		public String mName;
		public String mDesc;
		
		public NormalizedIdentifier(String aID, String aName, String aDesc)
		{	mID=aID;
			mName=aName;
			mDesc=aDesc;
		}

		@Override
		public boolean equals(Object obj)
		{	return obj.hashCode()==this.hashCode();
		}

		@Override
		public int hashCode()
		{	int code=100000;
			if(mID!=null)
			{	code|=mID.hashCode();
			}
			if(mName!=null)
			{	code|=mName.hashCode();
			}
			if(mDesc!=null)
			{	code|=mDesc.hashCode();
			}
			return code;
		}
		
	}

}
